from pyspark.sql import SparkSession
from pyspark.sql.types import StructType
from pyspark.sql.types import IntegerType
from pyspark.sql.types import StringType

if __name__ == "__main__":
    spark = SparkSession \
        .builder \
        .appName("spark-redis") \
        .config("spark.redis.host", "192.168.0.201") \
        .config("spark.redis.port", "6379") \
        .config("spark.redis.auth", "xxxx") \
        .getOrCreate()

    # *********** read the redis data with DataFrame *******************

    # Prepare the data via redis cli:
    #    hset customer:1 name John age 30
    #    hset customer:2 name Peter age 45

    schema = StructType()
    schema.add("id", data_type=IntegerType())
    schema.add("name", data_type=StringType())
    schema.add("age", data_type=IntegerType())

    # read data with providing an explicit schema
    spark.read.format("redis") \
        .schema(schema) \
        .option("keys.pattern", "customer:*") \
        .option("key.column", "id") \
        .load().show()

    # read data with automatically infer the schema
    spark.read.format("redis") \
        .schema(schema) \
        .option("keys.pattern", "customer:*") \
        .option("infer.schema", True) \
        .load().show()

    # *********** write the redis data with DataFrame *******************
    df = spark.createDataFrame([(1, 'John', 30), (2, 'Peter', 45)], schema)

    df.write.format("redis") \
        .option("table", "person_py") \
        .option("key.column", "id") \
        .mode("overwrite") \
        .save()

    # Read data via redis cli:
    # redis:6379> HGETALL person_py:1
    # age
    # 30
    # name
    # John

    spark.read.format("redis") \
        .option("table", "person_py") \
        .option("key.column", "id") \
        .load().show()

    spark.stop()
