package com.huaweicloud.dli;

import com.huaweicloud.dli.io.common.Column;
import com.huaweicloud.dli.io.read.DownloadJob;
import com.huaweicloud.dli.io.read.Reader;
import com.huaweicloud.dli.io.read.SchemaUtils;
import com.huaweicloud.dli.io.write.TableInfo;
import com.huaweicloud.dli.io.write.UploadJob;
import com.huaweicloud.dli.service.HandleRow;
import com.huaweicloud.dli.service.UpdateRow;
import com.huaweicloud.dli.task.ReaderTask;
import com.huaweicloud.dli.task.WriterTask;
import com.huaweicloud.dli.util.TunnelUtils;
import com.huaweicloud.sdk.core.auth.BasicCredentials;

import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import lombok.extern.slf4j.Slf4j;

@Slf4j
public class DliTunnelExample {
    private static int threadNum;

    private static ExecutorService threadPool;

    private static BasicCredentials credentials;

    private static String obsEndpoint;

    private static String dliEndpoint;

    public static void main(String[] args) {
        credentials = new BasicCredentials()
            .withProjectId("Your project id")
            .withAk("Your access key")
            .withSk("Your secret key");
        obsEndpoint = "OBS endpoint you want to access";
        dliEndpoint = "DLI endpoint you want to access";

        threadNum = 2; // the thread num which you will used on upload or download job
        threadPool = Executors.newFixedThreadPool(threadNum);

        String queueName = "Your queue name";
        String dbName = "Your database name";
        String tblName = "Your table name";

        String dataPath = "Your obs path to write tmp data";

        String op = "upload"; // operation: upload or download

        try {
            if (op.equals("upload")) {
                concurrentUpload(queueName, dataPath, dbName, tblName);
            } else {
                concurrentDownload(queueName, dbName, tblName);
            }
        } catch (Exception e) {
            log.error("Unexpected exception for DLI tunnel example", e);
        }

        threadPool.shutdown();
    }

    private static void concurrentUpload(
        String queueName,
        String dataPath,
        String dbName,
        String tblName) throws Exception {
        TableInfo tblInfo = new TableInfo(dbName, tblName);
        UploadJob uploadJob = new UploadJob(credentials, obsEndpoint, dliEndpoint, queueName, dataPath, tblInfo);
        List<Column> schema = getSchema();
        CountDownLatch cdl = new CountDownLatch(threadNum);
        int fullRecords = 100; // records num
        for (int i = 0; i < threadNum; i++) {
            WriterTask task = new WriterTask(
                uploadJob, schema, (int) Math.ceil((double) fullRecords / threadNum), new UpdateRow(), cdl);
            threadPool.submit(task);
        }
        cdl.await();
        TunnelUtils.waitUploadFinish(uploadJob, true);
    }

    private static List<Column> getSchema() {
        String schemaStr = "{\"type\":\"struct\",\"fields\":["
            + "{\"name\":\"c1\",\"type\":\"boolean\"},"
            + "{\"name\":\"c2\",\"type\":\"tinyint\"},"
            + "{\"name\":\"c3\",\"type\":\"smallint\"},"
            + "{\"name\":\"c4\",\"type\":\"int\"},"
            + "{\"name\":\"c5\",\"type\":\"bigint\"},"
            + "{\"name\":\"c6\",\"type\":\"float\"},"
            + "{\"name\":\"c7\",\"type\":\"double\"},"
            + "{\"name\":\"c8\",\"type\":\"decimal(10,2)\"},"
            + "{\"name\":\"c9\",\"type\":\"date\"},"
            + "{\"name\":\"c10\",\"type\":\"timestamp\"},"
            + "{\"name\":\"c11\",\"type\":\"string\"},"
            + "{\"name\":\"c12\",\"type\":{\"type\":\"array\",\"elementType\":\"int\"}},"
            + "{\"name\":\"c13\",\"type\":{\"type\":\"map\",\"keyType\":\"string\",\"valueType\":\"string\"}},"
            + "{\"name\":\"c14\",\"type\":{\"type\":\"struct\",\"fields\":["
            + "{\"name\":\"sc1\",\"type\":\"string\"},{\"name\":\"sc2\",\"type\":\"boolean\"}]}}]}";
        return SchemaUtils.getSchemaFromJson(schemaStr);
    }

    private static void concurrentDownload(String queueName, String dbName, String tblName) throws Exception {
        String sql = String.format("SELECT * FROM `%s`.`%s`", dbName, tblName);
        DownloadJob downloadJob = new DownloadJob(credentials, obsEndpoint, dliEndpoint, queueName, sql);
        TunnelUtils.waitDownloadFinish(downloadJob);
        List<Reader> readers = downloadJob.createReaders(threadNum);
        CountDownLatch cdl = new CountDownLatch(readers.size());
        readers.forEach(reader -> {
            ReaderTask task = new ReaderTask(reader, true, new HandleRow(), cdl);
            threadPool.submit(task);
        });
        cdl.await();
        for (Reader reader : readers) {
            reader.close();
        }
    }
}
