package com.huaweicloud.dli.io.common;

import java.util.Locale;

public abstract class DataType {
    protected String name;

    public abstract String getName();

    public enum TypeName {
        BOOLEAN,
        TINYINT,
        SMALLINT,
        INT,
        BIGINT,
        FLOAT,
        DOUBLE,
        DECIMAL,
        DATE,
        TIMESTAMP,
        STRING,
        BINARY,
        ARRAY,
        MAP,
        STRUCT,
        VOID;

        public static TypeName fromName(String name) {
            name = name.toLowerCase(Locale.ROOT);
            if (name.equals("boolean")) {
                return BOOLEAN;
            }
            if (name.equals("tinyint") || name.equals("byte")) {
                return TINYINT;
            }
            if (name.equals("smallint") || name.equals("short")) {
                return SMALLINT;
            }
            if (name.equals("int") || name.equals("integer")) {
                return INT;
            }
            if (name.equals("bigint") || name.equals("long")) {
                return BIGINT;
            }
            if (name.equals("float")) {
                return FLOAT;
            }
            if (name.equals("double")) {
                return DOUBLE;
            }
            if (name.startsWith("decimal")) {
                return DECIMAL;
            }
            if (name.equals("date")) {
                return DATE;
            }
            if (name.equals("timestamp")) {
                return TIMESTAMP;
            }
            if (name.equals("string") || name.startsWith("char") || name.startsWith("varchar")) {
                return STRING;
            }
            if (name.equals("binary")) {
                return BINARY;
            }
            if (name.startsWith("array")) {
                return ARRAY;
            }
            if (name.startsWith("map")) {
                return MAP;
            }
            if (name.startsWith("struct")) {
                return STRUCT;
            }
            if (name.equals("void") || name.equals("null")) {
                return VOID;
            }
            throw new IllegalArgumentException("Unsupported data type: " + name);
        }
    }
}
