package com.huaweicloud.dli.io.common;

import com.huaweicloud.sdk.dli.v1.DliClient;
import com.huaweicloud.sdk.dli.v1.model.CreateSqlJobRequest;
import com.huaweicloud.sdk.dli.v1.model.CreateSqlJobRequestBody;
import com.huaweicloud.sdk.dli.v1.model.CreateSqlJobResponse;
import com.huaweicloud.sdk.dli.v1.model.ShowSqlJobStatusRequest;
import com.huaweicloud.sdk.dli.v1.model.ShowSqlJobStatusResponse;

import lombok.extern.slf4j.Slf4j;

@Slf4j
public class DliProxy {
    private final DliClient client;

    private final String queueName;

    public DliProxy(DliClient client, String queueName) {
        this.client = client;
        this.queueName = queueName;
    }

    public String submitSql(String sql) {
        CreateSqlJobResponse resp = client.createSqlJob(
            new CreateSqlJobRequest().withBody(
                new CreateSqlJobRequestBody().withQueueName(queueName).withSql(sql)));
        if (!resp.getIsSuccess()) {
            throw new RuntimeException("Submit sql failed, details: " + resp.getMessage());
        }
        return resp.getJobId();
    }

    public ShowSqlJobStatusResponse getSqlJobStatus(String sqlJobId) {
        ShowSqlJobStatusResponse resp = client.showSqlJobStatus(
            new ShowSqlJobStatusRequest().withJobId(sqlJobId));
        if (!resp.getIsSuccess()) {
            throw new RuntimeException("Get sql job status failed, details: " + resp.getMessage());
        }
        return resp;
    }
}
