package com.huaweicloud.dli.io.common;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import lombok.Getter;

@Getter
public class Row {
    private final List<Column> schema;

    private final List<Object> record;

    public Row(List<Column> schema) {
        this.schema = schema;
        record = new ArrayList<>();
        for (int i = 0; i < schema.size(); i++) {
            record.add(null);
        }
    }

    public void setBoolean(int index, Boolean value) {
        validIndexAndDataType(index, DataType.TypeName.BOOLEAN);
        record.set(index, value);
    }

    public Boolean getBoolean(int index) {
        validIndexAndDataType(index, DataType.TypeName.BOOLEAN);
        return (Boolean) record.get(index);
    }

    public void setTinyint(int index, Byte value) {
        validIndexAndDataType(index, DataType.TypeName.TINYINT);
        record.set(index, value);
    }

    public Byte getTinyint(int index) {
        validIndexAndDataType(index, DataType.TypeName.TINYINT);
        return (Byte) record.get(index);
    }

    public void setSmallint(int index, Short value) {
        validIndexAndDataType(index, DataType.TypeName.SMALLINT);
        record.set(index, value);
    }

    public Short getSmallint(int index) {
        validIndexAndDataType(index, DataType.TypeName.SMALLINT);
        return (Short) record.get(index);
    }

    public void setInt(int index, Integer value) {
        validIndexAndDataType(index, DataType.TypeName.INT);
        record.set(index, value);
    }

    public Integer getInt(int index) {
        validIndexAndDataType(index, DataType.TypeName.INT);
        return (Integer) record.get(index);
    }

    public void setBigint(int index, Long value) {
        validIndexAndDataType(index, DataType.TypeName.BIGINT);
        record.set(index, value);
    }

    public Long getBigint(int index) {
        validIndexAndDataType(index, DataType.TypeName.BIGINT);
        return (Long) record.get(index);
    }

    public void setFloat(int index, Float value) {
        validIndexAndDataType(index, DataType.TypeName.FLOAT);
        record.set(index, value);
    }

    public Float getFloat(int index) {
        validIndexAndDataType(index, DataType.TypeName.FLOAT);
        return (Float) record.get(index);
    }

    public void setDouble(int index, Double value) {
        validIndexAndDataType(index, DataType.TypeName.DOUBLE);
        record.set(index, value);
    }

    public Double getDouble(int index) {
        validIndexAndDataType(index, DataType.TypeName.DOUBLE);
        return (Double) record.get(index);
    }

    public void setDate(int index, Date value) {
        validIndexAndDataType(index, DataType.TypeName.DATE);
        record.set(index, value);
    }

    public Date getDate(int index) {
        validIndexAndDataType(index, DataType.TypeName.DATE);
        return (Date) record.get(index);
    }

    public void setTimestamp(int index, Timestamp value) {
        validIndexAndDataType(index, DataType.TypeName.TIMESTAMP);
        record.set(index, value);
    }

    public Timestamp getTimestamp(int index) {
        validIndexAndDataType(index, DataType.TypeName.TIMESTAMP);
        return (Timestamp) record.get(index);
    }

    public void setString(int index, String value) {
        validIndexAndDataType(index, DataType.TypeName.STRING);
        record.set(index, value);
    }

    public String getString(int index) {
        validIndexAndDataType(index, DataType.TypeName.STRING);
        return (String) record.get(index);
    }

    public void setDecimal(int index, BigDecimal value) {
        validIndexAndDataType(index, DataType.TypeName.DECIMAL);
        record.set(index, value);
    }

    public BigDecimal getDecimal(int index) {
        validIndexAndDataType(index, DataType.TypeName.DECIMAL);
        return (BigDecimal) record.get(index);
    }

    public void setBinary(int index, byte[] value) {
        validIndexAndDataType(index, DataType.TypeName.BINARY);
        record.set(index, value);
    }

    public byte[] getBinary(int index) {
        validIndexAndDataType(index, DataType.TypeName.BINARY);
        return (byte[]) record.get(index);
    }

    private void validIndexAndDataType(int index, DataType.TypeName typeName) {
        if (index > schema.size()) {
            throw new IllegalArgumentException("Index exceed");
        }

        if (DataType.TypeName.fromName(schema.get(index).getType().getName()) != typeName) {
            throw new IllegalArgumentException("Incorrect data type");
        }
    }
}
