package com.huaweicloud.dli.io.read;

import com.huaweicloud.dli.io.common.Column;
import com.huaweicloud.dli.io.common.TunnelJob;
import com.huaweicloud.sdk.core.auth.BasicCredentials;
import com.huaweicloud.sdk.dli.v1.model.ShowSqlJobStatusResponse;
import com.obs.services.model.ObsObject;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

public class DownloadJob extends TunnelJob {
    private String resultPath;

    private List<Column> resultSchema;

    public DownloadJob(
        BasicCredentials credential,
        String obsEndpoint,
        String dliEndpoint,
        String queueName,
        String sql) {
        super(credential, obsEndpoint, dliEndpoint, queueName);
        this.sqlJobId = dliProxy.submitSql(sql);
    }

    public String getInnerJobStatus() {
        ShowSqlJobStatusResponse resp = dliProxy.getSqlJobStatus(sqlJobId);

        if (resp.getDetail() != null && !resp.getDetail().isEmpty() && resultSchema == null) {
            resultSchema = SchemaUtils.getSchemaFromJson(resp.getDetail());
        }

        if (resp.getResultPath() != null && !resp.getResultPath().isEmpty() && resultPath == null) {
            resultPath = resp.getResultPath();
        }

        return resp.getStatus().getValue();
    }

    public synchronized List<Reader> createReaders(int expectedMaxConcurrent) throws IOException {
        if (resultPath == null || resultPath.isEmpty()) {
            throw new RuntimeException("Missing sql job result path");
        }

        List<ObsObject> objects = obsProxy.listObjects(
            obsProxy.getBucketName(resultPath), obsProxy.getObjectPrefix(resultPath))
            .stream().filter(obsObj ->
                obsObj.getMetadata().getContentLength() > 0 && !obsObj.getObjectKey().endsWith("metadata.json"))
            .collect(Collectors.toList());
        int realConcurrent = Math.min(expectedMaxConcurrent, objects.size());
        List<Reader> readers = new ArrayList<>(realConcurrent);
        int step = (int) Math.ceil((double) objects.size() / realConcurrent);
        for (int i = 0; i < objects.size(); ) {
            readers.add(new CsvObsReader(obsProxy, copyList(objects, i, i + step), resultSchema));
            i += step;
        }
        return readers;
    }

    private List<ObsObject> copyList(List<ObsObject> originList, int start, int end) {
        List<ObsObject> newList = new ArrayList<>(end - start);
        int fEnd = Math.min(end, originList.size());
        for (int i = start; i < fEnd; i++) {
            newList.add(originList.get(i));
        }
        return newList;
    }
}
