package com.huaweicloud.dli.io.write;

import com.huaweicloud.dli.io.common.TunnelJob;
import com.huaweicloud.sdk.core.auth.BasicCredentials;

import java.io.IOException;
import java.util.Map;
import java.util.StringJoiner;

public class UploadJob extends TunnelJob {
    private final String dataPath;

    private final TableInfo tblInfo;

    public UploadJob(
        BasicCredentials credential,
        String obsEndpoint,
        String dliEndpoint,
        String queueName,
        String dataPath,
        TableInfo tblInfo) {
        super(credential, obsEndpoint, dliEndpoint, queueName);
        this.dataPath = dataPath;
        this.tblInfo = tblInfo;
    }

    public synchronized Writer createWriter() throws IOException {
        return new JsonObsWriter(obsProxy, dataPath);
    }

    public synchronized void beginCommit() throws IOException {
        sqlJobId = dliProxy.submitSql(genLoadSql());
    }

    private String genLoadSql() {
        String overwrite = tblInfo.isOverwrite() ? "OVERWRITE" : "";
        String partSpecClause = "";
        if (tblInfo.getPartSpec() != null && !tblInfo.getPartSpec().isEmpty()) {
            StringJoiner sj = new StringJoiner(", ");
            for(Map.Entry<String, String> entry : tblInfo.getPartSpec().entrySet()) {
                sj.add(String.format("`%s`='%s'", entry.getKey(), entry.getValue()));
            }
            partSpecClause = String.format("PARTITION (%s)", sj);
        }
        return String.format("LOAD DATA INPATH 'obs://%s/%s' %s INTO TABLE `%s`.`%s` %s OPTIONS(data_type 'json')",
            obsProxy.getBucketName(dataPath), obsProxy.getObjectPrefix(dataPath),
            overwrite, tblInfo.getDbName(), tblInfo.getTblName(), partSpecClause);
    }

    public String getInnerJobStatus() {
        return dliProxy.getSqlJobStatus(sqlJobId).getStatus().getValue();
    }
}
