package com.huaweicloud.dli.service;

import com.huaweicloud.dli.io.common.Column;
import com.huaweicloud.dli.io.common.DataType;
import com.huaweicloud.dli.io.common.Row;
import com.huaweicloud.sdk.core.utils.JsonUtils;

import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.function.Consumer;

public class HandleRow implements Consumer<Row> {
    private static final ThreadLocal<DateFormat> DATE_FORMAT =
        ThreadLocal.withInitial(() -> new SimpleDateFormat("yyyy-MM-dd"));

    private static final ThreadLocal<DateFormat> TIMESTAMP_FORMAT =
        ThreadLocal.withInitial(() -> new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSSZZ"));

    @Override
    public void accept(Row row) {
        // TODO: add your business logic to handle the row data
        List<Column> schema = row.getSchema();
        List<Object> record = row.getRecord();
        for (int i = 0; i < schema.size(); i++) {
            DataType.TypeName typeName = DataType.TypeName.fromName(schema.get(i).getType().getName());
            switch (typeName) {
                case STRING:
                    String strV = (String) record.get(i);
                    System.out.println(
                        "\t" + (strV == null ? null : strV.replaceAll("\r", "\\\\r").replaceAll("\n", "\\\\n")));
                    break;
                case DATE:
                    Date dtV = (Date) record.get(i);
                    System.out.println("\t" + (dtV == null ? null : DATE_FORMAT.get().format(dtV)));
                    break;
                case TIMESTAMP:
                    Timestamp tsV = (Timestamp) record.get(i);
                    System.out.println("\t" + (tsV == null ? null : TIMESTAMP_FORMAT.get().format(tsV)));
                    break;
                case ARRAY:
                case MAP:
                case STRUCT:
                    Object data = record.get(i);
                    System.out.println("\t" + (data == null ? null : JsonUtils.toJSON(data)));
                    break;
                default:
                    System.out.println("\t" + record.get(i));
            }
        }
        System.out.println("--------------------------------------------------------------------------------");
    }
}
