package com.huaweicloud.dli.service;

import com.huaweicloud.dli.io.common.Column;
import com.huaweicloud.dli.io.common.DataType;
import com.huaweicloud.dli.io.common.Row;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Consumer;

public class UpdateRow implements Consumer<Row> {
    @Override
    public void accept(Row row) {
        // TODO: add your business logic to update the row data
        List<Column> schema = row.getSchema();
        List<Object> record = row.getRecord();
        for (int i = 0; i < schema.size(); i++) {
            DataType.TypeName typeName = DataType.TypeName.fromName(schema.get(i).getType().getName());
            switch (typeName) {
                case BOOLEAN:
                    row.setBoolean(i, true);
                    break;
                case TINYINT:
                    row.setTinyint(i, (byte) 1);
                    break;
                case SMALLINT:
                    row.setSmallint(i, (short) 123);
                    break;
                case INT:
                    row.setInt(i, 65535);
                    break;
                case BIGINT:
                    row.setBigint(i, 123456789012L);
                    break;
                case FLOAT:
                    row.setFloat(i, 101.235f);
                    break;
                case DOUBLE:
                    row.setDouble(i, 256.012358);
                    break;
                case DECIMAL:
                    row.setDecimal(i, new BigDecimal("33.05")); // decimal(p, 2)
                    break;
                case DATE:
                    row.setDate(i, new Date(1683543480000L));
                    break;
                case TIMESTAMP:
                    row.setTimestamp(i, new Timestamp(1683543480000L));
                    break;
                case STRING:
                    row.setString(i, "abc_123&");
                    break;
                case ARRAY:
                    record.set(i, Arrays.asList(1, 2, 3));
                    break;
                case MAP:
                    Map<String, String> map = new HashMap<String, String>() {{
                        put("k1", "v1");
                    }};
                    record.set(i, map);
                    break;
                case STRUCT:
                    Map<String, Object> struct = new HashMap<String, Object>() {{
                        put("sc1", "Abc");
                        put("sc2", true);
                    }};
                    record.set(i, struct);
                    break;
                default:
                    throw new IllegalArgumentException("Unsupported data type: " + typeName.name());
            }
        }
    }
}
