package com.huaweicloud.dli.task;

import com.huaweicloud.dli.io.common.Column;
import com.huaweicloud.dli.io.common.Row;
import com.huaweicloud.dli.io.write.UploadJob;
import com.huaweicloud.dli.util.TunnelUtils;

import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.function.Consumer;

import lombok.extern.slf4j.Slf4j;

@Slf4j
public class WriterTask implements Runnable {
    private final UploadJob uploadJob;

    private final List<Column> schema;

    private final int recordNum;

    private final Consumer<Row> updateRow;

    private final CountDownLatch cdl;

    public WriterTask(
        UploadJob uploadJob,
        List<Column> schema,
        int recordNum,
        Consumer<Row> updateRow,
        CountDownLatch cdl) {
        this.uploadJob = uploadJob;
        this.schema = schema;
        this.recordNum = recordNum;
        this.updateRow = updateRow;
        this.cdl = cdl;
    }

    @Override
    public void run() {
        try {
            TunnelUtils.writeData(uploadJob, schema, recordNum, updateRow);
        } catch (Exception e) {
            log.error("Unexpected exception while write data", e);
        } finally {
            cdl.countDown();
        }
    }
}
