package com.huawei.dli.demo;

import org.apache.hadoop.hive.ql.exec.UDAF;
import org.apache.hadoop.hive.ql.exec.UDAFEvaluator;

public class AvgFilterUDAFDemo extends UDAF {

    /**
     * 定义静态内部类AvgFilter
     */
    public static class PartialResult {
        public Long sum;
    }

    public static class VarianceEvaluator implements UDAFEvaluator {

        // 初始化PartialResult对象
        private AvgFilterUDAFDemo.PartialResult partial;

        // 创建VarianceEvaluator无参构造函数
        public VarianceEvaluator(){
            this.partial = new AvgFilterUDAFDemo.PartialResult();
            this.partial.sum = 0L;
        }

        /**
         * init函数类似于构造函数，用于UDAF的初始化
         */
        @Override
        public void init() {

            // 设置sum初始值
            this.partial.sum = 0L;
        }

        /**
         * iterate接收传入的参数，并进行内部的轮转。
         * @param x
         * @return
         */
        public void iterate(Long x) {
            if (x == null) {
                return;
            }
            AvgFilterUDAFDemo.PartialResult temp = this.partial;
            temp.sum = temp.sum | x;
        }

        /**
         * terminatePartial无参数，其为iterate函数遍历结束后，返回轮转数据，
         * terminatePartial类似于hadoop的Combiner
         * @return
         */
        public AvgFilterUDAFDemo.PartialResult terminatePartial() {
            return this.partial;
        }

        /**
         * merge接收terminatePartial的返回结果，进行数据merge操作
         * @param
         * @return
         */
        public void merge(AvgFilterUDAFDemo.PartialResult pr) {
            if (pr == null) {
                return;
            }
            AvgFilterUDAFDemo.PartialResult temp = this.partial;
            temp.sum = temp.sum | pr.sum;
        }

        /**
         * terminate返回最终的聚集函数结果
         * @return
         */
        public Long terminate() {
            if (this.partial.sum == null) {
                return 0L;
            }
            return this.partial.sum;
        }
    }
}